==> devito-4.8.3.tar.gz: devito-4.8.3/devito/types/utils.py <==
from ctypes import POINTER, Structure

from devito.tools import EnrichedTuple, Tag
# Additional Function-related APIs

__all__ = ['Buffer', 'DimensionTuple', 'NODE', 'CELL', 'IgnoreDimSort',
           'HierarchyLayer', 'HostLayer']


class Buffer(Tag):

    def __init__(self, value):
        super(Buffer, self).__init__('Buffer', value)


class Stagger(Tag):
    """Stagger region."""
    pass

NODE = Stagger('node')  # noqa
CELL = Stagger('cell')


class DimensionTuple(EnrichedTuple):

    def __getitem_hook__(self, dim):
        for d in self._getters:
            if d._defines & dim._defines:
                return self._getters[d]
        raise KeyError


class IgnoreDimSort(tuple):
    """A tuple subclass used to wrap the implicit_dims to indicate
    that the topological sort of other dimensions should not occur."""
    pass


class CtypesFactory(object):

    cache = {}

    @classmethod
    def generate(cls, pname, pfields):
        key = (pname, tuple(pfields))
        try:
            return cls.cache[key]
        except KeyError:
            dtype = POINTER(type(pname, (Structure,), {'_fields_': pfields}))
            return cls.cache.setdefault(key, dtype)


class HierarchyLayer(object):

    """
    Represent a generic layer of the node storage hierarchy (e.g., disk, host).
    """

    def __init__(self, suffix=''):
        self.suffix = suffix

    def __repr__(self):
        return "Layer<%s>" % self.suffix

    def __eq__(self, other):
        return (isinstance(other, HierarchyLayer) and
                self.suffix == other.suffix)

    def __hash__(self):
        return hash(self.suffix)


class HostLayer(HierarchyLayer):
    pass

==> devito-4.8.5.tar.gz: devito-4.8.5/devito/types/utils.py <==
from ctypes import POINTER, Structure

from devito.tools import EnrichedTuple, Tag
# Additional Function-related APIs

__all__ = ['Buffer', 'DimensionTuple', 'NODE', 'CELL', 'IgnoreDimSort',
           'HierarchyLayer', 'HostLayer']


class Buffer(Tag):

    def __init__(self, value):
        super().__init__('Buffer', value)


class Stagger(Tag):
    """Stagger region."""
    pass

NODE = Stagger('node')  # noqa
CELL = Stagger('cell')


class DimensionTuple(EnrichedTuple):

    def __getitem_hook__(self, dim):
        for d in self._getters:
            if d._defines & dim._defines:
                return self._getters[d]
        raise KeyError


class IgnoreDimSort(tuple):
    """A tuple subclass used to wrap the implicit_dims to indicate
    that the topological sort of other dimensions should not occur."""
    pass


class CtypesFactory(object):

    cache = {}

    @classmethod
    def generate(cls, pname, pfields):
        key = (pname, tuple(pfields))
        try:
            return cls.cache[key]
        except KeyError:
            dtype = POINTER(type(pname, (Structure,), {'_fields_': pfields}))
            return cls.cache.setdefault(key, dtype)


class HierarchyLayer(object):

    """
    Represent a generic layer of the node storage hierarchy (e.g., disk, host).
    """

    def __init__(self, suffix=''):
        self.suffix = suffix

    def __repr__(self):
        return "Layer<%s>" % self.suffix

    def __eq__(self, other):
        return (isinstance(other, HierarchyLayer) and
                self.suffix == other.suffix)

    def __hash__(self):
        return hash(self.suffix)


class HostLayer(HierarchyLayer):
    pass
